package jq.qde;

import java.io.PrintStream;
import java.util.HashMap;
import java.util.Iterator;
import java.util.NoSuchElementException;
import jq.qde.space.QualitativeVariable;
import jq.qde.space.QuantitySpace;
import jq.qde.space.VariableQuantitySpace;
import jq.qde.space.value.QualitativeMagnitude;
import jq.qde.space.value.QualitativeValue;


/**
 * A list of variables with ordered sets of LandmarkValues.
 * SystemQuantitySpace is a constant class so that instances can be shared between states.
 * @author oursland
 */
public class QDEQuantitySpace {
	private final VariableQuantitySpace[] vqs;
	private final HashMap map = new HashMap();

//	private SystemQuantitySpace(HashMap map) {
//		this.map.putAll(map);
//	}

	public QDEQuantitySpace(VariableQuantitySpace[] vqs) {
		this.vqs = (VariableQuantitySpace[]) vqs.clone();
		for (int i = 0; i < vqs.length; i++) {
			map.put(vqs[i].variable, vqs[i]);
		}
	}

	public void printSpecification(PrintStream out, String prefix) {
		out.println(" (quantity-spaces");
		for (Iterator iter = map.values().iterator(); iter.hasNext();) {
			QuantitySpace vqs = (QuantitySpace) iter.next();
			vqs.printSpecification(out, prefix+" ");
			out.println();
		}
		out.println(" )");
	}

//	public Iterator variableIterator() {
//		return map.keySet().iterator();
//	}

	public QualitativeVariable getVariable(String name) {
		for (int i = 0; i < getVariableCount(); i++) {
			QualitativeVariable var = getVariable(i);
			if(var.getName().equals(name)) {
				return var;
			}
			
		}
		throw new NoSuchElementException(name);
	}

	public int getVariableCount() {
		return map.size();
	}

	public QualitativeVariable getVariable(int i) {
		return getVariableSpace(i).variable;
//		for (Iterator iter = variableIterator(); iter.hasNext();) {
//			QualitativeVariable var = (QualitativeVariable) iter.next();
//			if( i == 0 ) {
//				return var;
//			}
//			i--;
//		}
//		throw new ArrayIndexOutOfBoundsException(i);
	}

	public VariableQuantitySpace getVariableSpace(int i) {
		return this.vqs[i];
	}
	
	public VariableQuantitySpace getVariableSpace(QualitativeVariable var) {
		return (VariableQuantitySpace) map.get(var);		
	}

	public boolean isPositive(QualitativeMagnitude mag) {
		QuantitySpace varSpace = getVariableSpace(mag.getVariable());
		return varSpace.isPositive(mag);
	}

	public boolean isNegative(QualitativeMagnitude mag) {
		QuantitySpace varSpace = getVariableSpace(mag.getVariable());
		return varSpace.isNegative(mag);
	}

	public boolean isLowEnd(QualitativeMagnitude magnitude) {
		return getVariableSpace(magnitude.getVariable()).isLowEnd(magnitude);
	}

	public boolean isHighEnd(QualitativeMagnitude magnitude) {
		return getVariableSpace(magnitude.getVariable()).isHighEnd(magnitude);
	}
	
//	/**
//	 * Note that this does not change the target space. A new SystemQuantitySpace is created.
//	 * @param prevLM
//	 * @param newLM
//	 * @return
//	 */
//	public SystemQuantitySpace addAfter(ValueLandmark prevLM, ValueLandmark newLM) {
//		assert prevLM.getVariable() == newLM.getVariable();
//
//		final QualitativeVariable var = prevLM.getVariable();
//		final VariableQuantitySpace varSpace = getVariableSpace(var);
//		final HashMap map = new HashMap();
//		map.putAll(this.map);
//		map.put(var, varSpace.addAfter(prevLM, newLM));
//		return new SystemQuantitySpace(this.vqs);
//	}

	public QDEQuantitySpace add(VariableQuantitySpace vqs) {
		VariableQuantitySpace[] vqsa = new VariableQuantitySpace[this.vqs.length + 1];
		for (int i = 0; i < this.vqs.length; i++) {
			vqsa[i] = this.vqs[i];
		}
		vqsa[vqsa.length-1] = vqs;
		return new QDEQuantitySpace(vqsa);
	}
	
	/**
	 * This function should never return true for interval values.
	 * @param value The value to test for divergence.
	 * @return 'true' if the value is diverging (moving past the ends of the qspace). 'false' otherwise.
	 */
	public boolean isDiverging(QualitativeValue value) {
		return getVariableSpace(value.variable).isDiverging(value);
	}

	public boolean contains(VariableQuantitySpace space) {
		return map.get(space.variable) == space;
	}
}
