package jq.qde;

import java.io.PrintStream;
import oursland.naming.NamableObject;

import jq.qde.constraint.ConstraintCorrespondence;
import jq.qde.constraint.ConstraintValidationResult;
import jq.qde.space.QualitativeVariable;
import jq.qde.space.value.QualitativeValue;
import jq.qsim.QSimListener;

/**
 * @author oursland
 */
public abstract class QualitativeConstraint extends NamableObject {
	public static final ConstraintCorrespondence[] EMPTYCORR = {};
	
	private final QualitativeVariable[] args;
	private final ConstraintCorrespondence[] corr;
	
	public QualitativeConstraint(String name, QualitativeVariable[] args, ConstraintCorrespondence[] corr) { 
		super(name);
		this.args = args;
		this.corr = corr;
	}
	
//	public abstract boolean initConstraint(SystemState state);
//	public abstract boolean propagateConstraint(SystemState state, SystemStateChange change);
	
	final public boolean dependsOn(QualitativeVariable var) {
		for (int i = 0; i < args.length; i++) {
			if( args[i] == var ) {
				return true;
			}
		}
		return false;
	}

	final public void printSpecification(PrintStream out, String prefix) {
		out.print(prefix);
		out.print("((");
		out.print(getName());
		out.print(" ");
		for(int i = 0; i < getVariableCount(); i++ ) {
			if( i > 0 ) {
				out.print(" ");
			}
			out.print(getVariable(i).getName());
		}
		out.print(")");
		if( corr.length > 0 ) {
			for (int i = 0; i < corr.length; i++) {
				out.print(" ");
				corr[i].printSpecification(out, "");
			}		
		}
		out.print(")");		
	}
	
	final public int getVariableCount() {
		return args.length;
	}
	
	final public QualitativeVariable getVariable(int index) {
		return args[index];
	}

	final public int getCorrespondingValueCount() {
		return corr.length;
	}
	
	final public ConstraintCorrespondence getCorrespondingValue(int i) {
		return corr[i];
	}

	public boolean shouldCreateNewCorrespondence(QDEState state, QSimListener l) {
		if( !state.constraintValuesArePoints(this) ) {
			return false;
		}
		ConstraintCorrespondence matchCorr = getMatchingCorrespondence(state);
		if( matchCorr != null ) {
			l.existingCorrespondence(this, matchCorr);
			return false;
		}
		return true;
	}

	public boolean hasMatchingCorrespondingValue(QDEState state) {
		for( int j = 0; j < getCorrespondingValueCount(); j++ ) {
			if( getCorrespondingValue(j).matchesState(state) ) {
				return true;
			}
		}
		return false;
	}

	public ConstraintCorrespondence getMatchingCorrespondence(QDEState state) {
		for( int j = 0; j < getCorrespondingValueCount(); j++ ) {
			ConstraintCorrespondence corr = getCorrespondingValue(j);
			if( corr.matchesState(state) ) {
				return corr;
			}
		}
		return null;
	}

	public QualitativeConstraint addCorrespondingValue(ConstraintCorrespondence newCorr) {
		if( contains(newCorr) ) {
			return this;
		} else {
			ConstraintCorrespondence[] newCorrArray = new ConstraintCorrespondence[this.corr.length + 1];
			System.arraycopy(this.corr, 0, newCorrArray, 0, this.corr.length);
			newCorrArray[newCorrArray.length-1] = newCorr;
			return createCopy(this.args, newCorrArray);
		}
	}
	
	private boolean contains(ConstraintCorrespondence newCorr) {
		for (int i = 0; i < this.corr.length; i++) {
			if( this.corr[i].equals(newCorr)) {
				return true;
			}
		}
		return false;
	}

	public abstract ConstraintValidationResult validateValues(QualitativeValue[] stateArray);
	public abstract QualitativeConstraint createCopy(QualitativeVariable[] args, ConstraintCorrespondence[] corr);
}
