package jq.qde;

import java.io.PrintStream;
import java.util.ArrayList;
import java.util.Iterator;
import oursland.naming.NamableObject;

import jq.qde.space.QualitativeVariable;
import jq.qde.space.TimeQuantitySpace;
import jq.qde.space.TimeVariable;
import jq.qde.space.VariableQuantitySpace;

/**
 * @author oursland
 */
public class QualitativeDifferentialEquation extends NamableObject {
	public final TimeQuantitySpace tspace;
	public final QDEQuantitySpace qspace;
	private final QualitativeConstraint[] constraint;
	private final QualitativeTransition[] transitions;
	
	public QualitativeDifferentialEquation(String name) {
		super(name);
		this.qspace = new QDEQuantitySpace(new VariableQuantitySpace[0]);
		this.tspace = new TimeQuantitySpace(new TimeVariable("t_"));
		this.constraint = new QualitativeConstraint[0];
		this.transitions = new QualitativeTransition[0];
	}
	
	public QualitativeDifferentialEquation(String name, QDEQuantitySpace qspace, TimeQuantitySpace tspace, QualitativeConstraint[] constraints, QualitativeTransition[] transitions) {
		super(name);
		this.qspace = qspace;
		this.tspace = tspace;
		this.constraint = (QualitativeConstraint[]) constraints.clone();
		this.transitions = (QualitativeTransition[]) transitions.clone();
	}

	public QualitativeConstraint[] getConstraints() {
		return (QualitativeConstraint[]) constraint.clone();
	}

	public void printSpecification(PrintStream out, String prefix) {
		out.println(prefix+"(define-QDE " + getName());
		
		qspace.printSpecification(out, prefix+" ");
		
		out.println(prefix+" (constraints");
		for (int i = 0; i < constraint.length; i++) {
			constraint[i].printSpecification(out, prefix+"  ");
			out.println();
		}
		out.println(prefix+" )");
		
		if( transitions.length > 0 ) {
			out.println(prefix+" (transitions");
			for (int i = 0; i < transitions.length; i++) {
				transitions[i].printSpecification(out, prefix+"  ");
				out.println();
			}
			out.println(prefix+" )");
		}
		
		
		out.println(prefix+")");
	}

	public QDEQuantitySpace getQuantitySpace() {
		return qspace;
	}

	public QualitativeVariable getVariable(String name) {
		return qspace.getVariable(name);
	}

	public Iterator constraintIterator(QualitativeVariable var) {
		ArrayList rc = new ArrayList();
		for (int i = 0; i < constraint.length; i++) {
			if( constraint[i].dependsOn(var) ) {
				rc.add(constraint[i]);
			}
		}
		return rc.iterator();
	}

	public int getVariableCount() {
		return qspace.getVariableCount();
	}

	protected static final TimeQuantitySpace createTimeSpace() {
		return new TimeQuantitySpace(new TimeVariable("t_"));
	}

	public TimeQuantitySpace getTimeSpace() {
		return tspace;
	}
}
