package jq.qde.constraint;

import jq.qde.QDEState;
import jq.qde.QualitativeConstraint;
import jq.qde.space.QualitativeVariable;
import jq.qde.space.value.MagnitudeLandmark;
import jq.qde.space.value.QualitativeDirection;
import jq.qde.space.value.QualitativeValue;

/**
 * @author oursland
 */
public class MonotonicConstraint extends QualitativeConstraint {
	public final boolean increasing;

	private MonotonicConstraint(boolean increasing, QualitativeVariable[] args, ConstraintCorrespondence[] corr) {
		super("M" + (increasing ? "+" : "-"), args, corr);
		this.increasing = increasing;
	}

	public MonotonicConstraint(
		boolean increasing,
		QualitativeVariable x,
		QualitativeVariable y) {
		this(increasing, x, y, EMPTYCORR);	
	}
	
	public MonotonicConstraint(
		boolean increasing,
		QualitativeVariable x,
		QualitativeVariable y,
		ConstraintCorrespondence[] corr) {
		super("M" + (increasing ? "+" : "-"), new QualitativeVariable[] { x, y }, (ConstraintCorrespondence[]) corr.clone());
		this.increasing = increasing;
	}

//	public boolean propagateConstraint(SystemState state, SystemStateChange change) {
//		final QualitativeVariable var = change.getVariable();
//		final QualitativeVariable x = getArg(0);
//		final QualitativeVariable y = getArg(1);
//		if (change.isDirectionChange()) {
//			// propagate direction
//			if (x == var) {
//				return propagateDirection(state, x, y);
//			} else if (y == var) {
//				return propagateDirection(state, y, x);
//			}
//		} else if (change.isValueChange()) {
//			// use corresponding value constraints.
//			if (x == var) {
//				return propagateValue(state, x, y);
//			} else if (y == var) {
//				return propagateValue(state, y, x);
//			}
//		}
//		return false;
//	}

//	private boolean propagateValue(SystemState state, QualitativeVariable from, QualitativeVariable to) {
//		QualitativeMagnitude fromValue = state.getVariableMagnitude(from);
//		for (int i = 0; i < getCorrespondingValueCount(); i++) {
//			CorrespondingValue corr = getCorrespondingValue(i);
//			if (corr.getValue(from) == fromValue) {
//				QualitativeMagnitude toValue = state.getVariableMagnitude(to);
//				ValueLandmark nextValue = corr.getValue(to);
//				if (!toValue.isAssignable(nextValue)) {
//					return false;
//				}
//				if (nextValue != toValue) {
//					System.out.println(
//						"Propagating corresponding value from " + from.getName() + " to " + to.getName() + ".");
//					state.setVariableMagnitude(to, nextValue);
//				}
//			}
//		}
//		return true;
//	}

//	private boolean propagateDirection(SystemState state, QualitativeVariable from, QualitativeVariable to) {
//		QualitativeDirection nextDir = QualitativeDirection.INC;
//		QualitativeDirection fromDir = state.getVariableDirection(from);
//		if (fromDir == QualitativeDirection.INC && increasing) {
//			nextDir = QualitativeDirection.INC;
//		} else if (fromDir == QualitativeDirection.DEC && !increasing) {
//			nextDir = QualitativeDirection.INC;
//		} else if (fromDir == QualitativeDirection.INC && !increasing) {
//			nextDir = QualitativeDirection.DEC;
//		} else if (fromDir == QualitativeDirection.DEC && increasing) {
//			nextDir = QualitativeDirection.DEC;
//		} else if (fromDir == QualitativeDirection.STD) {
//			nextDir = QualitativeDirection.STD;
//		} else if (fromDir == QualitativeDirection.UNKNOWN) {
//			nextDir = QualitativeDirection.UNKNOWN;
//		}
//		QualitativeDirection toDir = state.getVariableDirection(to);
//		if (toDir == QualitativeDirection.UNKNOWN) {
//			System.out.println(
//				"Propagating monotonic direction from " + getArg(0).getName() + " to " + getArg(1).getName() + ".");
//			state.setVariableDirection(to, nextDir);
//		} else if (toDir != nextDir) {
//			return false;
//		}
//		return true;
//	}

	public boolean initConstraint(QDEState state) {
		return true;
	}

	public ConstraintValidationResult validateValues(QualitativeValue[] stateArray) {
		/*  Qualitative directions must match.  */
		final QualitativeValue xState = stateArray[0];
		final QualitativeValue yState = stateArray[1];
		
		final QualitativeDirection yDir = increasing?yState.direction:yState.direction.reverse();
		if (xState.direction != yDir) {
			return new ConstraintValidationResult(false, "qdir mismatch", this, stateArray);
		}

		/*  Loop over all corresponding values for this constraint */
		for( int i = 0; i < getCorrespondingValueCount(); i++ ) {
			final ConstraintCorrespondence corr = getCorrespondingValue(i);
			final MagnitudeLandmark xCorr = corr.getValue(stateArray[0]);
			final MagnitudeLandmark yCorr = corr.getValue(stateArray[1]);
			
			QualitativeDirection yCompare = increasing?yState.qspace.compare(yState.magnitude, yCorr):yState.qspace.compare(yState.magnitude, yCorr).reverse();
			if (xState.qspace.compare(xState.magnitude, xCorr) != yCompare) {
				return new ConstraintValidationResult(false, "cval mismatch", this, stateArray);
			}
		}
		return new ConstraintValidationResult(true, "OK", this, stateArray);
	}

	public QualitativeConstraint createCopy(QualitativeVariable[] args, ConstraintCorrespondence[] corr) {
		return new MonotonicConstraint(this.increasing, args, corr);
	}
}
