package jq.qde.space;

import java.util.HashMap;
import oursland.naming.NamableObject;

import jq.qde.space.value.InfiniteLandmark;
import jq.qde.space.value.MagnitudeInterval;
import jq.qde.space.value.MagnitudeLandmark;
import jq.qde.space.value.UnknownMagnitude;
import jq.qde.space.value.ZeroLandmark;
import jq.util.Abbrevable;

/**
 * Ordinal data can only be compared for order as well as equality.
 * @author oursland
 */
public class QualitativeVariable extends NamableObject implements Abbrevable {
	private final HashMap landmarkMap = new HashMap();
	private final HashMap intervalMap = new HashMap();

	private int suffix = 1;
	private boolean isIndependent = false;

	public final MagnitudeLandmark MINF = InfiniteLandmark.createMinusInf(this);
	public final MagnitudeLandmark ZERO = new ZeroLandmark(this);
	public final MagnitudeLandmark INF = InfiniteLandmark.createInf(this);
	public final MagnitudeLandmark UNKNOWN = new UnknownMagnitude(this);
	{
		landmarkMap.put(MINF.getName(), MINF);	
		landmarkMap.put(ZERO.getName(), ZERO);	
		landmarkMap.put(INF.getName(), INF);	
		landmarkMap.put(UNKNOWN.getName(), UNKNOWN);	
		landmarkMap.put("nil", UNKNOWN);	
		landmarkMap.put("?", UNKNOWN);	
	}
	
	public QualitativeVariable(String name) {
		super(name);
		this.fullName = name;
		this.prefix = name;
	}
	
	public boolean isIndependent() {
		return isIndependent;
	}

	public void setIndependent(boolean b) {
		isIndependent = b;
	}

	public MagnitudeLandmark getLandmark(String name) {
		MagnitudeLandmark rc = (MagnitudeLandmark) landmarkMap.get(name);
		if( rc == null ) {
			rc = createLandmark(name);
			landmarkMap.put(name, rc);
		}
		return rc;
	}
	
	protected MagnitudeLandmark createLandmark(String name) {
		return new MagnitudeLandmark(name, this);
	}
	
	public MagnitudeInterval getInterval(MagnitudeLandmark low, MagnitudeLandmark high) {
		assert low.getVariable() == this;
		assert high.getVariable() == this;
		MagnitudeInterval temp = createInterval(low, high);
		MagnitudeInterval rc = (MagnitudeInterval) intervalMap.get(temp);
		if( rc == null ) {
			rc = temp;
			intervalMap.put(rc, rc);
		}
		return rc;
	}

	protected MagnitudeInterval createInterval(MagnitudeLandmark low, MagnitudeLandmark high) {
		return new MagnitudeInterval(low, high);
	}

	public MagnitudeInterval getInterval(String low, String high) {
		return getInterval(getLandmark(low), getLandmark(high));
	}

	public MagnitudeLandmark[] getLandmarks(String[] lms) {
		MagnitudeLandmark[] rc = new MagnitudeLandmark[lms.length];
		for (int i = 0; i < rc.length; i++) {
			rc[i] = getLandmark(lms[i]);
		}
		return rc;
	}

	public MagnitudeLandmark createNewLandmark() {
		return getLandmark(getNextLandmarkName());
	}

	private String getNextLandmarkName() {
		return prefix + suffix++;
	}

	public String toSpecification() {
		return getName();
	}
	public String toAbbreviation() {
		return getName();
	}

	private String fullName;
	public void setFullName(String name) {
		this.fullName = name;
	}
	public String getFullName() {
		return this.fullName;
	}
	
	protected String prefix;
	public void setLandmarkPrefix(String prefix) {
		this.prefix = prefix;
	}
}
