package jq.qde.space.value;

import java.io.PrintStream;
import java.util.Arrays;
import jq.qde.space.QualitativeVariable;
import jq.qde.space.VariableQuantitySpace;


/**
 * @author oursland
 */
public class MagnitudeLandmark implements QualitativeMagnitude {
	private static final String[] reserved = {
		"?", "0", "dec", "inc", "inf", "minf", "std"
	};

	private final String name;
	private final QualitativeVariable var;
	
	public String toString() {
		return name;
	}

	/**
	 * ValueLandmarks should only be created through a Variable.getLandmark() call.
	 * @param name
	 * @param var
	 */
	public MagnitudeLandmark(String name, QualitativeVariable var) {
		if( this.getClass() == MagnitudeLandmark.class && Arrays.binarySearch(reserved, name) >= 0 ) {
			throw new Error("'" + name + "' is reserved for specialized types.");
		}
		this.name = name;
		this.var = var;//		this.sign = sign;
	}
	
	public void printSpecification(PrintStream out, String prefix) {
		out.print(prefix+getName());
	}
	
	public String toSpecification() {
		return getName();
	}
	
	public String toAbbreviation() {
		return getName();
	}
	
	public String getName() {
		return name;
	}

	public QualitativeVariable getVariable() {
		return var;
	}

	public boolean isZero() {
		return false;
	}

	public boolean isUnknown() {
		return false;
	}
	
	public boolean isInfinite() {
		return false;
	}

	public final boolean isInterval() {
		return false;
	}

	public final boolean isPoint() {
		return true;
	}

	public QualitativeDirection getInfiniteSign() {
		return QualitativeDirection.ZERO;
	}

	public QualitativeMagnitude[] getInitialValues(VariableQuantitySpace qspace) {
		return new QualitativeMagnitude[] { this };
	}
}
