package jq.qsim;

import java.io.PrintStream;
import java.util.HashMap;
import java.util.Iterator;
import java.util.Map;

import jq.util.Abbrevable;

/**
 * @author oursland
 */
public class ConstraintStatisticMap {
	private final Map statisticMap = new HashMap();
	

	public void setContext(Object context, Abbrevable target) {
		FilterStatistic stat = getStatistic(target);
		stat.context = context;
	}
	public void markContextUse(Object context, Abbrevable target) {
		FilterStatistic stat = getStatistic(target);
		stat.context = context;
		stat.usedByContext = true;
	}
	
	public void completeContextStatistics(Object constraint, QDEPossibleValues possibleValues) {
		incrementUseCounts(constraint);
		markUnusedAsBad(constraint);
		resetContexts();
	}

	private void incrementUseCounts(Object constraint) {
		Iterator iter = statisticMap.values().iterator();
		while(iter.hasNext()) {
			FilterStatistic stat = (FilterStatistic) iter.next();
			if( stat.context == constraint ) {
				if( stat.usedByContext ) {
					stat.constraintCount++;
				}
			}
		}
	}

	private void markUnusedAsBad(Object constraint) {
		Iterator iter = statisticMap.values().iterator();
		while(iter.hasNext()) {
			FilterStatistic stat = (FilterStatistic) iter.next();
			if( stat.context == constraint ) {
				if( !stat.usedByContext ) {
					stat.bad = true;
				}
			}
		}
	}

	private void resetContexts() {
		Iterator iter = statisticMap.values().iterator();
		while(iter.hasNext()) {
			FilterStatistic stat = (FilterStatistic) iter.next();
			stat.usedByContext = false;
			stat.context = null;
		}
	}
	
	public int getUseCount(Abbrevable target) {
		return getStatistic(target).constraintCount;
	}
	public void invalidateTarget(Abbrevable target) {
		getStatistic(target).bad = true;
	}
	public boolean isValid(Abbrevable target) {
		return !getStatistic(target).bad;
	}
	private FilterStatistic getStatistic(Abbrevable target) {
		FilterStatistic rc = (FilterStatistic) statisticMap.get(target);
		if( rc == null ) {
			rc = new FilterStatistic(target);
			statisticMap.put(target, rc);
		}
		return rc;
	}
	
	public static class FilterStatistic {
		Object context = null;	/* The constraint that is considering this state */
		boolean usedByContext = false;			/* Is the constraint in use */
		int  constraintCount = 0;				/* number of constraints accepting this pval */
		boolean bad = false;					/* Initially FALSE, but set TRUE if any constraint can't use this value. */
	
		final Object target;
		public FilterStatistic( Object target ) {
			this.target = target; 
		}
		
		public void print(PrintStream out) {
			out.print("bad="+bad+", inUse="+usedByContext+", useCount="+constraintCount);
		}
	}

	public void printStatistics(PrintStream out) {
		for (Iterator iter = statisticMap.entrySet().iterator(); iter.hasNext();) {
			Map.Entry element = (Map.Entry) iter.next();
			Abbrevable value = (Abbrevable) element.getKey();
			FilterStatistic stats = (FilterStatistic)element.getValue();

			System.out.print(value.toSpecification());
			System.out.print("\t");
			stats.print(System.out);
			System.out.println();
		}
	}
}
