package jq.qsim;

import java.util.HashMap;
import java.util.Iterator;

import jq.qde.QDEQuantitySpace;
import jq.qde.QDEState;
import jq.qde.space.QualitativeVariable;
import jq.qde.space.TimeQuantitySpace;
import jq.qde.space.value.QualitativeValue;
import jq.qde.space.value.TimeValue;

/**
 * @author oursland
 */
public class QDEPossibleValues {
	private final TimeValue time;
	private final TimeQuantitySpace tspace;
	private final QDEQuantitySpace qspace; 
	private final HashMap variableMap = new HashMap();

	public static QDEPossibleValues createInitial(QDEState state, QSimListener listener) {
		QDEPossibleValues rc = new QDEPossibleValues(state.getTime(), state);
		rc.createInitialTuples(state, listener);
		return rc;
	}
	
	public static QDEPossibleValues createNext(QDEState state, QSimListener listener) {
		QDEPossibleValues rc = new QDEPossibleValues(state.getNextTime(), state);
		rc.createNextTuples(state, listener);
		return rc;
	}
	
	private QDEPossibleValues(TimeValue time, QDEState state) {
		this.qspace = state.qspace;
		this.tspace = state.tspace;
		this.time = time;
	}

	public QDEQuantitySpace getQuantitySpace() {
		return this.qspace;
	}

	public TimeQuantitySpace getTimeSpace() {
		return this.tspace;
	}
	
	public Iterator valueIterator(QualitativeVariable var) {
		return getVariablePossibilities(var).valueIterator();
	}
	
	public void removeState(QualitativeValue state) {
		VariablePossibleValues varStates = (VariablePossibleValues) variableMap.get(state.getVariable());
		varStates.removeState(state);
	}

	public int getPossibleValueCount(QualitativeVariable var) {
		VariablePossibleValues states = (VariablePossibleValues) variableMap.get(var);
		if (states != null) {
			return states.getPossibleValueCount();
		}
		throw new Error("Variable \"" + var.getName() + "\" not in system state.");
	}

	private VariablePossibleValues getVariablePossibilities(QualitativeVariable variable) {
		return (VariablePossibleValues) variableMap.get(variable);
	}
	
	public TimeValue getTime() {
		return time;
	}

	private void createInitialTuples(QDEState state, QSimListener l) {
		for( int i = 0; i < this.qspace.getVariableCount(); i++ ) {
			QualitativeVariable var = this.qspace.getVariable(i);
			VariablePossibleValues ivals = state.getInitialValues(var);
			variableMap.put(var, ivals);
		}
		notifyIVals(l);
	}

	private void createNextTuples(QDEState state, QSimListener l) {
		for( int i = 0; i < this.qspace.getVariableCount(); i++ ) {
			QualitativeVariable var = this.qspace.getVariable(i);
			VariablePossibleValues pvals = state.getPossibleValues(var, l);
			variableMap.put(var, pvals);
		}
		notifyPVals(l);
	}

	private void notifyIVals(QSimListener l) {
		for( int i = 0; i < this.qspace.getVariableCount(); i++ ) {
			QualitativeVariable var = this.qspace.getVariable(i);
			VariablePossibleValues ivals = (VariablePossibleValues) variableMap.get(var);
			l.initialValues(ivals);
		}
	}

	private void notifyPVals(QSimListener l) {
		for( int i = 0; i < this.qspace.getVariableCount(); i++ ) {
			QualitativeVariable var = this.qspace.getVariable(i);
			VariablePossibleValues pvals = (VariablePossibleValues) variableMap.get(var);
			l.possibleValues(pvals);
		}
	}

	public VariablePossibleValues getPossibleValues(QualitativeVariable var) {
		return (VariablePossibleValues) variableMap.get(var);
	}
}
