package jq.qsim;

import java.util.ArrayList;
import java.util.List;
import oursland.naming.UIDGenerator;
import jq.qde.QualitativeDifferentialEquation;
import jq.qde.QDEState;

/**
 * @author oursland
 */
public class QSimUtil {
	public static void doSimulation(QualitativeDifferentialEquation qde, QDEState state, List resultList, QSimListener listener) {
		UIDGenerator gen = new UIDGenerator();
//		SimulationPrinter listener = new SimulationPrinter(System.out);
		
		ArrayList initialStates = new ArrayList();
		QSimUtil.doInitialState(qde, state, gen, initialStates, listener);
		
		final int maxStates = 80;
		ArrayList currentStateList = initialStates;
		int currentStates = currentStateList.size();
		while(!currentStateList.isEmpty()) {
			resultList.addAll(currentStateList);
			QDEState[] currentSetArray = (QDEState[]) currentStateList.toArray(new QDEState[currentStateList.size()]);
			currentStateList.clear();
			for (int i = 0; i < currentSetArray.length; i++) {
				QSimUtil.doNextStates(currentSetArray[i], gen, currentStateList, listener);
			}
			currentStates += currentStateList.size();
			if( currentStates > maxStates ) {
				System.out.println("Processed the maximum number of states.");
				break;
			}
		}
		
//		PrintUtil.printCreatedStates(listener.stateFormatter, (SystemState[]) totalSetList.toArray(new SystemState[currentStateList.size()]));
	}

	public static void doInitialState(QualitativeDifferentialEquation qde, QDEState state, UIDGenerator gen, List resultList, QSimListener listener) {
//		System.out.println("Finding Initial States:");

		QDEPossibleValues ivals = state.getInitialValues(listener);
//		PrintUtil.printPossibleValues(ivals);

		QSimFilter filter = new QSimFilter(ivals, qde.getConstraints());
		filter.createTuples(listener);
//		PrintUtil.printConstraintTuples(filter);

		filter.filterTuples(listener);
//		PrintUtil.printFilterResults(filter);

		filter.formStates(gen, resultList, listener);		
//		PrintUtil.printCreatedStates(initialStates);
	}

	public static void doNextStates(QDEState parentState, UIDGenerator gen, List resultList, QSimListener listener) {
		if( parentState.isQuiescent() ) {
			listener.quiescentState(parentState);
			return;
		}else if( parentState.isDiverging() ) {
			listener.divergingState(parentState);
			return;
		} else {
			listener.startPrediction(parentState);
//			parentState.print(System.out, "\t");
		
			QSimFilter filter = new QSimFilter(parentState, listener);
//			PrintUtil.printPossibleValues(filter.possibleValues);
		
			filter.createTuples(listener);
//			PrintUtil.printConstraintTuples(filter);
		
			filter.filterTuples(listener);
//			PrintUtil.printFilterResults(filter);
		
			filter.formStates(gen, resultList, listener);
//			PrintUtil.printCreatedStates(nextStates);

			// note: filtering has side-effects. It removes some of the tuples. Not all tuples will be printed.
		
			return;			
		}
//		return new SystemState[0];
	}
}
